#!/bin/sh
# mnt2rogld.sh - script to create a read-only root system on target user ID
# Hard-coded virtual device addresses:
#  1b0 - /boot
#  1b1 - /
#  1b2 - swap (VDISK)
#  1b3 - swap (VDISK)
#  1b4 - swap 
#  1b5 - /local
#  1b6 - /var/lib/rpm
#
# Source disks are linked as 1xxx
# Target disks are linked as 2xxx
#
# IBM DOES NOT WARRANT OR REPRESENT THAT THE CODE PROVIDED IS COMPLETE
# OR UP-TO-DATE.  IBM DOES NOT WARRANT,  REPRESENT OR IMPLY RELIABILITY,
# SERVICEABILITY OR FUNCTION OF THE CODE.  IBM IS UNDER NO OBLIGATION TO
# UPDATE  CONTENT NOR PROVIDE FURTHER SUPPORT.
# ALL CODE IS PROVIDED "AS IS," WITH NO WARRANTIES OR GUARANTEES WHATSOEVER.
# IBM EXPRESSLY DISCLAIMS TO THE  FULLEST EXTENT PERMITTED BY LAW ALL EXPRESS,
# IMPLIED,  STATUTORY AND OTHER WARRANTIES, GUARANTEES, OR  REPRESENTATIONS,
# INCLUDING, WITHOUT LIMITATION, THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR  PURPOSE, AND NON-INFRINGEMENT OF PROPRIETARY AND INTELLECTUAL
# PROPERTY RIGHTS.  YOU UNDERSTAND AND AGREE THAT  YOU USE THESE MATERIALS,
# INFORMATION, PRODUCTS, SOFTWARE, PROGRAMS, AND SERVICES, AT YOUR OWN
# DISCRETION AND  RISK AND THAT YOU WILL BE SOLELY RESPONSIBLE FOR ANY DAMAGES
# THAT MAY RESULT, INCLUDING LOSS OF DATA OR DAMAGE TO YOUR COMPUTER SYSTEM.
# IN NO EVENT WILL IBM BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY OR  CONSEQUENTIAL DAMAGES OF ANY TYPE
# WHATSOEVER RELATED TO OR ARISING FROM USE OF THE CODE FOUND HEREIN, WITHOUT
# LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOST SAVINGS, LOSS OF
# PROGRAMS OR OTHER DATA, EVEN IF IBM IS EXPRESSLY ADVISED OF THE POSSIBILITY
# OF SUCH DAMAGES. THIS EXCLUSION AND WAIVER OF LIABILITY APPLIES TO ALL
# CAUSES OF ACTION, WHETHER BASED ON CONTRACT, WARRANTY, TORT OR ANY OTHER
# LEGAL THEORIES.
 
#+--------------------------------------------------------------------------+
function CPcmd()
# Run a CP command and invoke it via the vmcp module/command
#   Arg1-n: the command to issue
#   Return: the command's return code
#+--------------------------------------------------------------------------+
 {
  echo "Invoking CP command: $@"
# parse output to get return code: awk -F# splits line at '#' with rc at end
  output=$(vmcp $@ 2>&1)
  if [ "X$output" != "X" ]; then # there is some output - echo it
    echo "$output"
  fi
  retVal=0
  retVal=$(echo $output | grep "Error: non-zero CP" | awk -F# '{print $2}')
  return $retVal
 }

#+--------------------------------------------------------------------------+
function checkID()
#   Arg 1: User ID to verify     
# Verify user ID exists and is logged off
#+--------------------------------------------------------------------------+
 {
  userID=$1
  CPcmd QUERY $userID
  rc=$?
  case $rc in
    0)  # user ID is logged on or disconnected
      echo "Error: $userID user ID must be logged off"
      exit 2
      ;;
    3)  # user ID does not exist
      echo "Error: $ID user ID does not exist"
      exit 3
      ;;
   45) # user ID is logged off - this is correct - fall through
      ;;
   *) # unexpected
      echo "Unexpected rc from QUERY: $rc"
      echo "$targetID user ID must exist and be logged off"
      exit 4
  esac
 }

#+--------------------------------------------------------------------------+
function linkSourceDisks() 
# Link source disks as 1xxx 
#+--------------------------------------------------------------------------+
 {
  echo ""
  echo "Linking source disks ..."
  CPcmd link $sourceID 1b1 11b1 rr 
  if [ $? != 0 ]; then exit 5; fi
  CPcmd link $sourceID 1b0 11b0 rr
  if [ $? != 0 ]; then exit 6; fi
  CPcmd link $sourceID 1b4 11b4 rr
  if [ $? != 0 ]; then exit 7; fi
  CPcmd link $sourceID 1b5 11b5 rr
  if [ $? != 0 ]; then exit 8; fi
  CPcmd link $sourceID 1b6 11b6 rr
  if [ $? != 0 ]; then exit 9; fi
 }

#+--------------------------------------------------------------------------+
function linkTargetDisks() 
# Link the target disks as 2xxx 
#+--------------------------------------------------------------------------+
 {
  echo ""
  echo "Linking target disks ..."
  CPcmd link $targetID 1b1 21b1 mr 
  if [ $? != 0 ]; then exit 10; fi
  CPcmd link $targetID 1b0 21b0 mr
  if [ $? != 0 ]; then exit 11; fi
  CPcmd link $targetID 1b4 21b4 mr
  if [ $? != 0 ]; then exit 12; fi
  CPcmd link $targetID 1b5 21b5 mr
  if [ $? != 0 ]; then exit 13; fi
  CPcmd link $targetID 1b6 21b6 mr
  if [ $? != 0 ]; then exit 14; fi
 }

#+--------------------------------------------------------------------------+
function enableSourceDisks() 
# Enable the source disks
#+--------------------------------------------------------------------------+
 {
  echo ""
  echo "Enabling source disks ..."
  chccwdev -e 11b1
  if [ $? != 0 ]; then exit 15; fi
  chccwdev -e 11b0
  if [ $? != 0 ]; then exit 16; fi
  chccwdev -e 11b4
  if [ $? != 0 ]; then exit 17; fi
  chccwdev -e 11b5
  if [ $? != 0 ]; then exit 18; fi
  chccwdev -e 11b6
  if [ $? != 0 ]; then exit 19; fi
  udevsettle
 }

#+--------------------------------------------------------------------------+
function enableTargetDisks() 
# Enable the target disks
#+--------------------------------------------------------------------------+
 {
  echo ""
  echo "Enabling target disks ..."
  chccwdev -e 21b1
  if [ $? != 0 ]; then exit 20; fi
  chccwdev -e 21b0
  if [ $? != 0 ]; then exit 21; fi
  chccwdev -e 21b4
  if [ $? != 0 ]; then exit 22; fi
  chccwdev -e 21b5
  if [ $? != 0 ]; then exit 23; fi
  chccwdev -e 21b6
  if [ $? != 0 ]; then exit 24; fi
  udevsettle
 }

#+--------------------------------------------------------------------------+
function copyDisk() 
# Copy a disk via dasdfmt then dd 
#   Arg 1: Source vaddr
#   Arg 2: Target vaddr
#+--------------------------------------------------------------------------+
 {
  source=$1
  target=$2
  echo ""
  echo "copying $source to $target ..." 
  sDev=/dev/$(egrep ^0.0.$source /proc/dasd/devices | awk '{ print $7 }')
  if [ "$?" != 0 ]; then exit 25; fi
  tDev=/dev/$(egrep ^0.0.$target /proc/dasd/devices | awk '{ print $7 }')
  if [ "$?" != 0 ]; then exit 26; fi
  echo ""
  echo "dasdfmt-ing $tDev ..."
  dasdfmt -d cdl -y -b 4096 -p -F -f $tDev
  if [ "$?" != 0 ]; then exit 27; fi
  echo ""
  echo "dd-ing $sDev to $tDev ..."
  dd bs=4096 if=$sDev of=$tDev
  if [ "$?" != 0 ]; then exit 28; fi
  echo ""
  echo "disabling and re-enabling $target ..."
  blockdev --rereadpt $tDev
  if [ $? != 0 ]; then exit 29; fi
  sync
  if [ $? != 0 ]; then exit 30; fi
  udevsettle
  if [ $? != 0 ]; then exit 31; fi
 }

#+--------------------------------------------------------------------------+
function copySystem()
# Copy /mnt/source to /mnt/target 
#+--------------------------------------------------------------------------+
 {
  echo ""
  echo "Copying root file system ..."
  copyDisk 11b1 21b1

  echo ""
  echo "Copying /boot/ ..."
  copyDisk 11b0 21b0

  echo ""
  echo "Copying minidisk swap space ..."
  copyDisk 11b4 21b4

  echo ""
  echo "Copying /local/ ..."
  copyDisk 11b5 21b5

  echo ""
  echo "Copying /var/lib/rpm/ ..."
  copyDisk 11b6 21b6
 }

#+--------------------------------------------------------------------------+
function mountSourceRoot()
# Mount disk at 11b1 over /mnt/source
#+--------------------------------------------------------------------------+
 {
  echo ""
  echo "Making source mount point ..."
  if [ ! -d /mnt/source ]; then
    mkdir /mnt/source
    if [ "$?" != 0 ]; then exit 32; fi
  fi
  echo ""
  echo "Mounting source root file system over /mnt/source ..."
  dev11b1=/dev/$(egrep '^0.0.11b1'  /proc/dasd/devices | awk '{ print $7 }')1
  mount -o ro $dev11b1 /mnt/source
 }

#+--------------------------------------------------------------------------+
function mountTargetDisks() 
# Mount disk at 21b1 over /mnt/target, then make mount points. 
# Then mount disks at 21b0 over /boot and 21b5 over /local
#+--------------------------------------------------------------------------+
 {
  dev21b0=/dev/$(egrep '^0.0.21b0'  /proc/dasd/devices | awk '{ print $7 }')
  dev21b1=/dev/$(egrep '^0.0.21b1'  /proc/dasd/devices | awk '{ print $7 }')
  dev21b5=/dev/$(egrep '^0.0.21b5'  /proc/dasd/devices | awk '{ print $7 }')
  dev21b6=/dev/$(egrep '^0.0.21b6'  /proc/dasd/devices | awk '{ print $7 }')

  echo ""
  echo "Mounting target file systems over /mnt/target ..."
  mkdir /mnt/target
  mount "$dev21b1"1 /mnt/target
  if [ "$?" != 0 ]; then exit 33; fi
  echo ""
  echo "Making target mount points ..."
  mkdir -p /mnt/target/{boot,usr,var,opt,local,sys,proc}
  mount "$dev21b0"1 /mnt/target/boot
  if [ "$?" != 0 ]; then exit 34; fi
  mount "$dev21b5"1 /mnt/target/local
  if [ "$?" != 0 ]; then exit 35; fi
  # label disk
  e2label "$dev21b5"1 stateless-state
  echo "Mounting memory file systems ..."
  mount -t sysfs sysfs /mnt/target/sys
  if [ "$?" != 0 ]; then exit 36; fi
  mount -t proc proc /mnt/target/proc
  if [ "$?" != 0 ]; then exit 37; fi
 }

#+--------------------------------------------------------------------------+
function modifySystem()
# 1) Modify /etc/modprobe.conf and run mkinitrd on target system
# 2) Copy source r/w directories to target /local/
# 3) Configure target system for read-only /
#+--------------------------------------------------------------------------+
 {
  TGT="/mnt/target"
  echo ""
  echo "Backing up and modifying /etc/modprobe.conf file ..."
  cp $TGT/etc/modprobe.conf $TGT/etc/modprobe.conf.orig
  if [ "$?" != 0 ]; then exit 38; fi
  cat $TGT/etc/modprobe.conf.orig | \
   sed -e 's/1b0-1bf/1b0(ro),1b1(ro),1b2-1b5,1b6(ro),1b7-1bf/g' > \
   $TGT/etc/modprobe.conf

  echo ""
  echo "Backing up and modifying /etc/fstab file ..."
  cp $TGT/etc/fstab $TGT/etc/fstab.orig
  if [ "$?" != 0 ]; then exit 39; fi
  echo "tmpfs                   /tmp                    tmpfs   defaults        0 0" >> $TGT/etc/fstab

  echo ""
  echo "Running mkinitrd in target environment ..."
  # yes, this is ugly code:
  ramdisk=`ls /mnt/target/boot/initrd-*`
  unameimg=${ramdisk##/mnt/target/boot/initrd-}
  unamer=${unameimg%%.img}
  chroot $TGT mkinitrd --with dasd_eckd_mod --with dasd_fba_mod -f /boot/initrd-${unamer}.img $unamer
  if [ "$?" != 0 ]; then exit 40; fi

  echo ""
  echo "Backing up and modifying /etc/zipl.conf file ..."
  cp $TGT/etc/zipl.conf $TGT/etc/zipl.conf.orig
  if [ "$?" != 0 ]; then exit 41; fi
  cat $TGT/etc/zipl.conf.orig | \
   sed -e 's/vmpoff=LOGOFF/vmpoff=LOGOFF readonlyroot/g' > $TGT/etc/zipl.conf
  echo "Running zipl in target environment ..."
  chroot $TGT zipl
  if [ "$?" != 0 ]; then exit 42; fi
  
  echo ""
  echo "Backing up and modifying /etc/sysconfig/readonly-root file ..."
  cp $TGT/etc/sysconfig/readonly-root $TGT/etc/sysconfig/readonly-root.orig
  sed -i 's/READONLY=no/READONLY=yes/' $TGT/etc/sysconfig/readonly-root
  if [ "$?" != 0 ]; then exit 43; fi
  sed -i 's/STATE_MOUNT=\/.snapshot/STATE_MOUNT=\/local/' $TGT/etc/sysconfig/readonly-root
  if [ "$?" != 0 ]; then exit 44; fi

  # Add code to /etc/rc.d/rc.local on target system to set hostname 
  # This is to work around issue with the host name being set *before* 
  # /local/etc is bind-mounted over /etc)
  grepCmd="grep HOSTNAME /etc/sysconfig/network|awk -F= '{print \$2}'"
  echo "tgtHost=\`$grepCmd\`"  >> $TGT/etc/rc.d/rc.local
  echo 'hostname $tgtHost' >> $TGT/etc/rc.d/rc.local
  if [ "$?" != 0 ]; then exit 45; fi

  echo ""
  echo "Copying target /etc,root,srv,var/ to target /local/ ..." 
  cp -a $TGT/etc $TGT/local 2>/dev/null
  if [ "$?" != 0 ]; then exit 46; fi
  cp -a $TGT/root $TGT/local 2>/dev/null
  if [ "$?" != 0 ]; then exit 47; fi
  cp -a $TGT/srv $TGT/local 2>/dev/null
  if [ "$?" != 0 ]; then exit 48; fi
  cp -a $TGT/var $TGT/local 2>/dev/null
  if [ "$?" != 0 ]; then exit 49; fi
  echo /etc > $TGT/local/files
  echo /var >> $TGT/local/files
  echo /srv >> $TGT/local/files
  echo /root >> $TGT/local/files
 }

#+--------------------------------------------------------------------------+
function cleanUp()
# Unmount source and target file systems and detach minidisks 
#+--------------------------------------------------------------------------+
 {
  echo ""
  echo "Cleaning up ..."
  # clean up target disks
  umount /mnt/target/local
  umount /mnt/target/boot
  umount /mnt/target/proc
  umount /mnt/target/sys
  umount /mnt/target
  chccwdev -d 21b0
  chccwdev -d 21b1
  chccwdev -d 21b4
  chccwdev -d 21b5
  chccwdev -d 21b6
  vmcp det 21b0
  vmcp det 21b1
  vmcp det 21b4
  vmcp det 21b5
  vmcp det 21b6
  # clean up source disks
  umount /mnt/source
  chccwdev -d 11b0
  chccwdev -d 11b1
  chccwdev -d 11b4
  chccwdev -d 11b5
  chccwdev -d 11b6
  vmcp det 11b0
  vmcp det 11b1
  vmcp det 11b4
  vmcp det 11b5
  vmcp det 11b6
 }

# main()
# global variables
sourceID="RH5RWMNT"
targetID="RH5ROGLD"
fstabFile="fstab.RH5"

# function calls
checkID $sourceID
checkID $targetID
linkSourceDisks
linkTargetDisks
enableSourceDisks
enableTargetDisks
copySystem
mountSourceRoot
mountTargetDisks
modifySystem
cleanUp
echo "Exiting with rc 0!"
exit 0
